/*
	Copyright 2006-2007 Geoffrey Miche <gmi@info.fundp.ac.be> (Facults Universitaires Notre-Dame de la Paix)

	This research work is co-founded by the European Commission under the Leonardo framework (V3 project, agreement
	number B/05/B/F/PP-144.005, http://www.leonardo-v3.eu).  V3, which stands for "Versatile and educational Vocational
	training Vehicle", is aiming at designing, implementing, deploying, evaluating and exploiting a versatile, 
	userfriendly, ICT-based Vocational and Educational Training (VET) framework suited for both young people in initial
	training as well as adult in lifelong learning activities.
	The V3 project partnership is composed of : 
		- Facults Universitaires Notre-Dame de la Paix (Namur, Belgium)
		- ICN groupe (Nancy, France)
		- Centre de Recherche Public Henri Tudor (Luxemburg)
		- Universitt Bremen (Bremen, Germany)
		- EMMERCE EEIG (Sweden)
		- Masarykova Univerzita (Brno, Czech Republic)

	This source code is licensed under Open Software license v3.0.  You can find a copy of this license at the 
	following URL : http://www.opensource.org/licenses/osl-3.0.php.  This notice has to be retained in all derived works.
*/
#include "stdafx.h"
#include "MergeStatusWnd.h"

MergeStatusWnd::MergeStatusWnd(CWnd *ParentWnd, LPCTSTR WndTitle, int areaHeight, int areaWidth, int autonaming){
	this->areaHeight = areaHeight;
	this->areaWidth = areaWidth;
	Create(ParentWnd, WndTitle, autonaming);
}

MergeStatusWnd::~MergeStatusWnd(){
	CWnd::DestroyWindow();
}

BOOL MergeStatusWnd::Create(CWnd *ParentWnd, LPCTSTR WndTitle, int autonaming){
	BOOL result = FALSE;
	
	//Window Class registration
	CString WndClassName = AfxRegisterWndClass(CS_OWNDC | CS_PARENTDC | CS_NOCLOSE,
											   ::LoadCursor(NULL, IDC_APPSTARTING),
											   CBrush(::GetSysColor(COLOR_BTNFACE)));
	
	 // Get the system window message font for use in the cancel button and text area
    NONCLIENTMETRICS ncm;
    ncm.cbSize = sizeof(NONCLIENTMETRICS);
    VERIFY(SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS), &ncm, 0));
    font.CreateFontIndirect(&(ncm.lfMessageFont)); 

	//Create the window
	DWORD propertyWnd = 0;
	if (autonaming == 0)
		propertyWnd = WS_EX_DLGMODALFRAME | WS_EX_NOPARENTNOTIFY;
	else
		propertyWnd = WS_EX_DLGMODALFRAME | WS_EX_NOPARENTNOTIFY | WS_EX_TOPMOST;
	result = CreateEx(propertyWnd,
					  WndClassName,
					  WndTitle,
					  WS_POPUP | WS_BORDER | WS_CAPTION,
					  0,0,
					  this->areaWidth,this->areaHeight,
					  ParentWnd->GetSafeHwnd(),
					  0,
					  NULL);
	if (!result) return FALSE;

	//Create the window static control
	CRect rectangle = new CRect(0,0,10,10);
	result = text.Create(_T(""), 
						WS_CHILD | WS_VISIBLE | SS_NOPREFIX | SS_LEFTNOWORDWRAP,
						rectangle,
						this,
						125);
	if (!result) return FALSE;
	text.SetFont(&font, TRUE);
	
	//Create the progress bar control
	result = WndProgress.Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH,
								rectangle,
								this,
								126);
	if (!result) return FALSE;
	WndProgress.SetRange(0,100);
	WndProgress.SetPos(0);
	SetAreaSize(this->areaHeight, this->areaWidth);
	CenterWindow();
	//RedrawWindow(NULL,NULL,RDW_ERASE | RDW_FRAME | RDW_INVALIDATE | RDW_UPDATENOW);
	this->ShowWindow(SW_SHOW);

	return TRUE;
}

void MergeStatusWnd::SetAreaSize(int areaHeight, int areaWidth){
	int margin = 20;
	CRect textAreaRect, progressBarRect;
	
	//Setting default size for the text area in case of problems
	textAreaRect.SetRect(margin, margin, areaWidth - 2*margin, areaHeight + 2*margin);

	CDC *mergeWndDC = GetDC();
	if (mergeWndDC != NULL){
		CFont *oldFont = mergeWndDC->SelectObject(&font);
		CString str = _T("M");
		for (int i = 0; i < areaHeight-1; i++)
				str += _T("\nM");
		mergeWndDC->DrawText(str, textAreaRect, DT_CALCRECT | DT_NOCLIP | DT_NOPREFIX | DT_VCENTER);
		textAreaRect.right = textAreaRect.left + areaWidth;
		mergeWndDC->SelectObject(oldFont);
		ReleaseDC(mergeWndDC);
	}
	progressBarRect.SetRect(textAreaRect.left, textAreaRect.bottom + margin, textAreaRect.right, 
		textAreaRect.bottom + 2*margin);
	
	CSize *ClientSize = new CSize (margin + textAreaRect.Width() + margin,
								   2*margin + textAreaRect.Height() + margin + progressBarRect.Height() + 2*margin);
	CRect WndRect, ClientRect;
	GetWindowRect(WndRect);
	GetClientRect(ClientRect);
	WndRect.right = WndRect.left + WndRect.Width() - ClientRect.Width() + ClientSize->cx;
	WndRect.bottom = WndRect.top + WndRect.Height() - ClientRect.Height() + ClientSize->cy;
	MoveWindow(WndRect);
	WndProgress.MoveWindow(progressBarRect);
	text.MoveWindow(textAreaRect);
}

void MergeStatusWnd::SetText(CString WndText){
	text.SetWindowText(WndText);
}

void MergeStatusWnd::UpdateBarPos(int position){
	if (position < 0)
		position = 0;
	else if (position > 100)
		position = 100;
	WndProgress.SetPos(position);
}

////////////////////////////////////////////////////////
//MESSAGE_MAP

BEGIN_MESSAGE_MAP(MergeStatusWnd, CWnd)
    //{{AFX_MSG_MAP(MergeStatusWnd)
    ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL MergeStatusWnd::OnEraseBkgnd(CDC* mergeWndDC){
    // Fill background with background colour
    CBrush brush(GetSysColor(COLOR_BTNFACE));
    CBrush* oldBrush = mergeWndDC->SelectObject(&brush);
    CRect rect;
	//Erasing the area not colored
    mergeWndDC->GetClipBox(&rect);
    mergeWndDC->PatBlt(rect.left, rect.top, rect.Width(), rect.Height(), PATCOPY);
    mergeWndDC->SelectObject(oldBrush);

    return TRUE;
}