// SignalDetector.cpp: implementation of the SignalDetector class.
//
/*	
	Copyright 2006-2007 Zednek Kotolan (Mazarikova Univerzita)

	This research work is co-founded by the European Commission under the Leonardo framework (V3 project, agreement
	number B/05/B/F/PP-144.005, http://www.leonardo-v3.eu).  V3, which stands for "Versatile and educational Vocational
	training Vehicle", is aiming at designing, implementing, deploying, evaluating and exploiting a versatile, 
	userfriendly, ICT-based Vocational and Educational Training (VET) framework suited for both young people in initial
	training as well as adult in lifelong learning activities.
	The V3 project partnership is composed of : 
		- Facults Universitaires Notre-Dame de la Paix (Namur, Belgium)
		- ICN groupe (Nancy, France)
		- Centre de Recherche Public Henri Tudor (Luxemburg)
		- Universitt Bremen (Bremen, Germany)
		- EMMERCE EEIG (Sweden)
		- Masarykova Univerzita (Brno, Czech Republic)

	This source code is licensed under Open Software license v3.0.  You can find a copy of this license at the 
	following URL : http://www.opensource.org/licenses/osl-3.0.php.  This notice has to be retained in all derived works.
*/

#include "../stdafx.h"
#include "SignalDetector.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSignalDetector::CSignalDetector()
{
	Buffer = NULL;
	IsSignal = true;
	BufferLength = -1;
	BufferIsCreated = false;
	BufferFull = false;
	Threshold = 0.05; //initialization of the border in %
	BufferPointer = 0;
    SamplingRate = 22050;
	Channels = 1;
	BPS = 16;
}

CSignalDetector::~CSignalDetector()
{
	FreeBuffer();
    IsSignal = true;
}

void CSignalDetector::SetSignalTrue(){
	if (!IsSignal) IsSignal = true;
}

void CSignalDetector::FreeBuffer(){
	if( BufferIsCreated ){
      delete [] Buffer;
	  BufferLength = -1;
      BufferIsCreated = false;
	  BufferFull = false;
	  BufferPointer = 0;
	}
}

void CSignalDetector::SetDetector(int in_BufferLength)
{ 
	// it depends on the type of the sampling rate
	FreeBuffer();
	Buffer = new float [in_BufferLength];
	BufferPointer = 0;
	BufferLength = in_BufferLength;
}

void CSignalDetector::SetThreshold(double in_Threshold)
{
	Threshold = in_Threshold;
}

void CSignalDetector::SetRecordingParams(int in_SamplingRate, int in_Channels, int in_BPS)
{
    SamplingRate = in_SamplingRate; //isn`t used
	Channels = in_Channels;
	BPS = in_BPS;
}

void CSignalDetector::WriteToBuffer(float Mean)
{ 
	// mean ~ energy E of sample / (number of samples in buffer)
    Buffer[BufferPointer] = Mean;
    BufferPointer++;
	if( BufferPointer >= BufferLength ){
		BufferPointer = 0;
		BufferFull = true;
	}
}

void CSignalDetector::ProcessBuffer(CBuffer* buffer)
{
	float mean, sum, max;
	if(Channels == 1){
		if(BPS == 8){
			sum = 0; 
			for(int i=0; i<buffer->ByteLen; i++){
              sum = sum + abs(buffer->ptr.b[i]-127);
			}
			mean = (float)((sum / buffer->ByteLen) / 1.27);
		}

		if(BPS == 16){
			sum = 0; 
			for(int i=0; i<buffer->ByteLen/2; i++){
              sum = sum + abs(buffer->ptr.s[i]);
			} 
			mean = (float)((sum / (buffer->ByteLen/2)) / 327.67);
		}

	}//if mono


    WriteToBuffer(mean);


	if(BufferFull){
		sum = 0;
		max = 0;
		for(int i=0; i<BufferLength; i++){
          sum = sum + Buffer[i];
		  if(max < Buffer[i]){
			  max = Buffer[i];
		  }//if max
		}//for buffer
	    mean = sum / BufferLength;

		IsSignal = (max > Threshold);
        //IsSignal = (mean > Threshold);
	}//if buffer full
}

bool CSignalDetector::IsSignalDetected()
{
  return IsSignal;
}

float CSignalDetector::GetActualMean()
{
	if( BufferPointer == 0 ){
        return Buffer[BufferLength-1];
	}else{
        return Buffer[BufferPointer-1];
	}

}