'	
'	Copyright 2006-2007 Geoffrey Miche <gmi@info.fundp.ac.be> (Facults Universitaires Notre-Dame de la Paix)
'                       Kaled Hadj Belgacem (Cogelis - ICN Business School)
'
'	This research work is co-founded by the European Commission under the Leonardo framework (V3 project, agreement
'	number B/05/B/F/PP-144.005, http://www.leonardo-v3.eu).  V3, which stands for "Versatile and educational Vocational
'	training Vehicle", is aiming at designing, implementing, deploying, evaluating and exploiting a versatile, 
'	userfriendly, ICT-based Vocational and Educational Training (VET) framework suited for both young people in initial
'	training as well as adult in lifelong learning activities.
'	The V3 project partnership is composed of : 
'		- Facults Universitaires Notre-Dame de la Paix (Namur, Belgium)
'		- ICN groupe (Nancy, France)
'		- Centre de Recherche Public Henri Tudor (Luxemburg)
'		- Universitt Bremen (Bremen, Germany)
'		- EMMERCE EEIG (Sweden)
'		- Masarykova Univerzita (Brno, Czech Republic)
'
'	This source code is licensed under Open Software license v3.0.  You can find a copy of this license at the 
'	following URL : http://www.opensource.org/licenses/osl-3.0.php.  This notice has to be retained in all derived works.
'
Imports System.Net.Sockets
Imports System.net
Imports System.IO
Imports System.Threading
Imports System.Text

Public Class ComSocketUdp

    Implements IDisposable

#Region ""
    'StateObject essentiel pour l'imbrication multi-mmorielle des processus pseudoforkes par le framework
    Public Class StateObject
        Public workSocket As Socket = Nothing
        Public BufferSize As Integer = 1024
        Public buffer(1024) As Byte
        Public sb As New StringBuilder
    End Class 'StateObject
#End Region

#Region " Variables "
    Dim _IP_string As String
    Dim _IP_Addr As IPAddress

    Dim _port_local As Integer
    Dim _ep_local As EndPoint 'point de terminaison local
    Dim vbSock_recep As Socket

    Dim _port_dest As Integer
    Dim _ep_dest As IPEndPoint 'destination des donnes

    Dim _port_ecoute As Integer
    Dim _ep_ecoute As IPEndPoint 'serveur distant  couter
#End Region

#Region " Constructeur "
    Public Sub New(ByVal inIp As String, ByVal inPort_ecoute As Integer, ByVal inPort_dest As Integer, Optional ByVal inPort_local As Integer = 0)
        _IP_string = inIp
        _IP_Addr = IPAddress.Parse(_IP_string)

        'port = 10000 pour CamStudio
        _port_ecoute = inPort_ecoute
        _ep_ecoute = New IPEndPoint(_IP_Addr, _port_ecoute)

        'port = 10000 pour CamStudio
        _port_dest = inPort_dest
        _ep_dest = New IPEndPoint(_IP_Addr, _port_dest)

        'port = 0 par dfaut (random)
        _port_local = inPort_local
        _ep_local = New IPEndPoint(IPAddress.Loopback, _port_local)
    End Sub
#End Region

#Region " Libration mmoire "
    '*********************************
    '*Mthode de libration de l'objet 
    '*en mmoire
    '*********************************
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overridable Overloads Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            ' Free other state (managed objects).
            _IP_string = Nothing
            _IP_Addr = Nothing

            _port_ecoute = Nothing
            _ep_ecoute = Nothing
            vbSock_recep = Nothing

            _port_dest = Nothing
            _ep_dest = Nothing

            _ep_local = Nothing
        End If
        ' Free your own state (unmanaged objects).
        ' Set large fields to null.
    End Sub

    Protected Overrides Sub Finalize()
        ' Simply call Dispose(False).
        Dispose(False)
    End Sub
#End Region

#Region " Proprits "
    Public Property IP_distante() As String
        Get
            Return _IP_string
        End Get
        Set(ByVal Value As String)
            _IP_string = Value
            'mise  jour de l'adresse IP / EndPoints de communication
            _IP_Addr = IPAddress.Parse(_IP_string)
            _ep_ecoute = New IPEndPoint(_IP_Addr, _port_ecoute)
            _ep_dest = New IPEndPoint(_IP_Addr, _port_dest)
            _ep_local = New IPEndPoint(_IP_Addr, _port_local)
        End Set
    End Property

    Public Property Port_ecoute() As Integer
        Get
            Return _port_ecoute
        End Get
        Set(ByVal Value As Integer)
            _port_ecoute = Value
            'mise  jour EndPoint coute
            _ep_ecoute = New IPEndPoint(_IP_Addr, _port_ecoute)
        End Set
    End Property

    Public Property Port_dest() As Integer
        Get
            Return _port_dest
        End Get
        Set(ByVal Value As Integer)
            _port_dest = Value
            'mise  jour EndPoint dest
            _ep_dest = New IPEndPoint(_IP_Addr, _port_dest)
        End Set
    End Property

    Public Property Port_local() As Integer
        Get
            Return _port_local
        End Get
        Set(ByVal Value As Integer)
            _port_local = Value
            'mise  jour EndPoint local
            _ep_local = New IPEndPoint(_IP_Addr, _port_local)
        End Set
    End Property
#End Region

#Region " Mthodes "
    Public Function Recevoir() As String
        Dim msg_byte() As Byte = Array.CreateInstance(Type.GetType("System.Byte"), 1024)
        Dim s_message As String = Nothing

        Try
            vbSock_recep = New Socket(_IP_Addr.AddressFamily, SocketType.Dgram, ProtocolType.Udp)
            vbSock_recep.Bind(_ep_ecoute)
            vbSock_recep.ReceiveFrom(msg_byte, _ep_local)
            vbSock_recep.Close()
        Catch ex As Exception
            Return ex.Message
        End Try

        s_message = System.Text.Encoding.UTF8.GetString(msg_byte)

        'Retraitement de la chaine envoye pour suprimmer les bytes inutiles
        Dim charTmp As Byte
        Dim i As Integer

        For i = 0 To 1023
            charTmp = msg_byte(i)
            If charTmp = 0 Then Exit For 'fin de la chaine => sortie
        Next

        s_message = s_message.Substring(0, i)

        Return s_message

    End Function

    Public Sub StopperReception()
        'on ferme le socket de rception
        vbSock_recep.Close()
    End Sub

    Public Function Envoyer(ByVal message As String) As String
        Dim msg_byte() As Byte = System.Text.Encoding.UTF8.GetBytes(message.ToCharArray)
        Dim vbSock As Socket

        Try
            vbSock = New Socket(_IP_Addr.AddressFamily, SocketType.Dgram, ProtocolType.Udp)
            vbSock.SendTo(msg_byte, 0, msg_byte.Length, SocketFlags.None, _ep_dest)
            vbSock.Close()

            Return Nothing
        Catch ex As Exception
            Return ex.Message
        End Try

    End Function
#End Region

End Class
